import bpy
import time
from bpy.props import (
    BoolProperty,
    PointerProperty,
    CollectionProperty,
    StringProperty,
    IntProperty,
    FloatProperty,
)
from bpy.types import Operator, Panel, PropertyGroup, UIList, AddonPreferences


# Updated object types list to include all 17 types
object_types = [
    ('MESH', 'assign_mesh', "Mesh"),
    ('CURVE', 'assign_curve', "Curve"),
    ('SURFACE', 'assign_surface', "Surface"),
    ('META', 'assign_meta', "Meta"),
    ('FONT', 'assign_font', "Font"),
    ('VOLUME', 'assign_volume', "Volume"),
    ('GPENCIL', 'assign_gpencil', "Grease Pencil"),
    ('ARMATURE', 'assign_armature', "Armature"),
    ('LATTICE', 'assign_lattice', "Lattice"),
    ('EMPTY', 'assign_empty', "Empty"),
    ('IMAGE', 'assign_image', "Image"),
    ('LIGHT', 'assign_light', "Light"),
    ('LIGHT_PROBE', 'assign_light_probe', "Light Probe"),
    ('CAMERA', 'assign_camera', "Camera"),
    ('SPEAKER', 'assign_speaker', "Speaker"),
    ('FORCE_FIELD', 'assign_force_field', "Force Field"),
    ('COLLECTION_INSTANCE', 'assign_collection_instance', "Collection Instance"),
]


# Function to get collection names (now includes new types)
def get_collection_names():
    """Return collection names in English."""
    return {
        'MESH': 'Mesh',
        'CURVE': 'Curve',
        'SURFACE': 'Surface',
        'META': 'Meta',
        'FONT': 'Font',
        'VOLUME': 'Volume',
        'GPENCIL': 'Grease Pencil',
        'ARMATURE': 'Armature',
        'LATTICE': 'Lattice',
        'EMPTY': 'Empty',
        'IMAGE': 'Image',
        'LIGHT': 'Light',
        'LIGHT_PROBE': 'Light Probe',
        'CAMERA': 'Camera',
        'SPEAKER': 'Speaker',
        'FORCE_FIELD': 'Force Field',
        'COLLECTION_INSTANCE': 'Collection Instance',
    }


def create_type_property(prop_name, display_name):
    def update(self, context):
        # Ensure only one parent collection has this type assigned
        if getattr(self, prop_name):
            for pcoll in context.scene.organize_properties.parent_collections:
                if pcoll != self:
                    if getattr(pcoll, prop_name):
                        setattr(pcoll, prop_name, False)
    return BoolProperty(name=display_name, update=update)


class ParentCollectionPropertyGroup(PropertyGroup):
    name: StringProperty(name="Collection Name", default="New Collection")

    # Dynamically create properties
    pass  # We'll add properties via __annotations__


# Ensure __annotations__ exists
if not hasattr(ParentCollectionPropertyGroup, '__annotations__'):
    ParentCollectionPropertyGroup.__annotations__ = {}

# Add type properties to the PropertyGroup with update functions
for obj_type, prop_name, display_name in object_types:
    prop = create_type_property(prop_name, display_name)
    ParentCollectionPropertyGroup.__annotations__[prop_name] = prop


class MaterialSelectionPropertyGroup(PropertyGroup):
    material_name: StringProperty(name="Material Name")
    select: BoolProperty(name="Select", default=False)


class ORGANIZE_OT_OrganizeCollections(Operator):
    """Organize objects into collections based on type, material properties, and parent-child relationships, ensuring each type is assigned to only one parent collection."""
    bl_idname = "object.organize_collections"
    bl_label = "Organize Collections"
    bl_options = {'REGISTER', 'UNDO'}

    _timer = None
    _start_time = None
    _duration = 3
    _steps = []
    _current_step = 0
    _created_collections = set()
    _existing_collections_before = set()
    _cancelled = False

    def execute(self, context):
        try:
            self.enable_emission_to_light = context.scene.organize_properties.enable_emission_to_light
            self.organize_selected_only = context.scene.organize_properties.organize_selected_only
            self.big_collection_active = context.scene.organize_properties.big_collection_active
            self.parent_child_active = context.scene.organize_properties.parent_child_active
            self.material_based_grouping_active = context.scene.organize_properties.material_based_grouping_active

            # Clean empty collections before starting a new organization
            self.clean_empty_collections()

            self._existing_collections_before = set(bpy.data.collections.keys())
            self._prepare_steps(context)

            total_steps = len(self._steps)
            # Retrieve dynamic display time from addon preferences
            addon_prefs = context.preferences.addons[__package__].preferences
            self._duration = min(addon_prefs.dynamic_display_time, total_steps * 0.1) if total_steps > 0 else addon_prefs.dynamic_display_time

            self._start_time = time.time()
            self._timer = context.window_manager.event_timer_add(0.1, window=context.window)
            context.window_manager.modal_handler_add(self)

            # Initialize progress bar
            context.window_manager.progress_begin(0, total_steps)

            return {'RUNNING_MODAL'}
        except Exception as e:
            self.report({'ERROR'}, "Error during organization: " + str(e))
            return {'CANCELLED'}

    def clean_empty_collections(self):
        """Remove empty collections from the scene."""
        collections_to_delete = []
        for collection in bpy.data.collections:
            if collection.name == bpy.context.scene.collection.name:
                continue  # Skip the master collection
            if not collection.objects and not collection.children:
                collections_to_delete.append(collection)

        for collection in collections_to_delete:
            try:
                # Unlink from all parent collections
                for parent in bpy.data.collections:
                    if collection.name in parent.children.keys():
                        parent.children.unlink(collection)
                # Remove collection
                bpy.data.collections.remove(collection)
            except Exception as e:
                self.report({'ERROR'}, f"Failed to delete collection '{collection.name}': {str(e)}")

    def _prepare_steps(self, context):
        self._steps = []
        self._created_collections = set()
        type_based_objects = {}
        parent_child_objects = {}
        material_based_objects = {}

        supported_types = {item[0] for item in object_types}

        objs_to_process = bpy.context.selected_objects if self.organize_selected_only else bpy.data.objects

        # Gather selected materials if material-based grouping is active
        selected_materials = set()
        if self.material_based_grouping_active:
            for mat_sel in context.scene.organize_properties.selected_materials:
                if mat_sel.select:
                    selected_materials.add(mat_sel.material_name)

        # Set to keep track of already assigned objects to prevent conflicts
        assigned_objects = set()

        for obj in objs_to_process:
            if obj is None:
                continue  # Skip if the object is None

            obj_type = obj.type

            # Handle collection instances and force fields
            if obj_type == 'EMPTY' and getattr(obj, 'instance_type', None) == 'COLLECTION':
                obj_type = 'COLLECTION_INSTANCE'
            elif hasattr(obj, 'field') and obj.field is not None and obj.field.type != 'NONE':
                obj_type = 'FORCE_FIELD'

            if obj_type == 'MESH' and is_single_emission_material(obj):
                if self.enable_emission_to_light:
                    obj_type = 'LIGHT'

            if obj_type not in supported_types:
                continue

            # Parent-Child Organization
            if self.parent_child_active and obj.parent:
                parent = obj.parent
                if parent not in parent_child_objects:
                    parent_child_objects[parent] = []
                parent_child_objects[parent].append(obj)
            else:
                if obj_type not in type_based_objects:
                    type_based_objects[obj_type] = []
                type_based_objects[obj_type].append(obj)

            # Material-Based Grouping
            if self.material_based_grouping_active and hasattr(obj.data, 'materials'):
                materials = obj.data.materials
                if materials:
                    for mat in materials:
                        if mat and mat.name in selected_materials:
                            mat_name = mat.name
                            if mat_name not in material_based_objects:
                                material_based_objects[mat_name] = []
                            material_based_objects[mat_name].append(obj)

        # Get the collection names (English only)
        collection_names = get_collection_names()

        # Priority 1: Material-Based Grouping
        if self.material_based_grouping_active:
            for mat_name, objs in material_based_objects.items():
                for obj in objs:
                    if obj.name in assigned_objects:
                        continue  # Skip already assigned objects
                    material_collection_name = f"Material_{mat_name}"
                    if material_collection_name not in bpy.data.collections:
                        try:
                            material_collection = bpy.data.collections.new(material_collection_name)
                            context.scene.collection.children.link(material_collection)
                            self._steps.append(('create_collection', material_collection_name))
                            self._created_collections.add(material_collection_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create material collection " + f"{material_collection_name}: " + str(e))
                            return
                    else:
                        material_collection = bpy.data.collections.get(material_collection_name)

                    if material_collection:
                        self._steps.append(('assign_object', material_collection_name, obj.name))
                        assigned_objects.add(obj.name)

        # Priority 2: Parent-Child Organization
        if self.parent_child_active:
            for parent, children in parent_child_objects.items():
                # Check if parent is already assigned via material-based grouping
                if parent.name in assigned_objects:
                    continue  # Skip if parent is already assigned

                parent_type = parent.type

                # Determine the big collection for the parent
                big_collection_name = None
                if self.big_collection_active:
                    for pcoll in context.scene.organize_properties.parent_collections:
                        for obj_type, prop_name, display_name in object_types:
                            if getattr(pcoll, prop_name) and parent_type == obj_type:
                                big_collection_name = pcoll.name
                                break
                        if big_collection_name:
                            break

                if big_collection_name:
                    if big_collection_name not in bpy.data.collections:
                        try:
                            big_collection = bpy.data.collections.new(big_collection_name)
                            context.scene.collection.children.link(big_collection)
                            self._steps.append(('create_collection', big_collection_name))
                            self._created_collections.add(big_collection_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create collection " + f"{big_collection_name}: " + str(e))
                            return
                    else:
                        big_collection = bpy.data.collections.get(big_collection_name)

                    # Create a group collection for the parent
                    parent_group_name = f"{parent.name}_Group"
                    if parent_group_name not in bpy.data.collections:
                        try:
                            parent_group = bpy.data.collections.new(parent_group_name)
                            big_collection.children.link(parent_group)
                            self._steps.append(('create_collection', parent_group_name, big_collection_name))
                            self._created_collections.add(parent_group_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create collection " + f"{parent_group_name}: " + str(e))
                            return
                    else:
                        parent_group = bpy.data.collections.get(parent_group_name)

                    # Assign parent and children to the parent_group
                    if parent.name not in assigned_objects:
                        self._steps.append(('assign_object', parent_group_name, parent.name))
                        assigned_objects.add(parent.name)
                    for child in children:
                        if child.name not in assigned_objects:
                            self._steps.append(('assign_object', parent_group_name, child.name))
                            assigned_objects.add(child.name)
                else:
                    # If big_collection_active is not enabled, assign to scene collection
                    parent_group_name = f"{parent.name}_Group"
                    if parent_group_name not in bpy.data.collections:
                        try:
                            parent_group = bpy.data.collections.new(parent_group_name)
                            context.scene.collection.children.link(parent_group)
                            self._steps.append(('create_collection', parent_group_name))
                            self._created_collections.add(parent_group_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create collection " + f"{parent_group_name}: " + str(e))
                            return
                    else:
                        parent_group = bpy.data.collections.get(parent_group_name)

                    # Assign parent and children to the parent_group
                    if parent.name not in assigned_objects:
                        self._steps.append(('assign_object', parent_group_name, parent.name))
                        assigned_objects.add(parent.name)
                    for child in children:
                        if child.name not in assigned_objects:
                            self._steps.append(('assign_object', parent_group_name, child.name))
                            assigned_objects.add(child.name)

        # Priority 3: Type-Based Organization
        if self.big_collection_active:
            # Collect types assigned to big collections
            assigned_types = {}
            for pcoll in context.scene.organize_properties.parent_collections:
                big_collection_name = pcoll.name
                for obj_type, prop_name, display_name in object_types:
                    if getattr(pcoll, prop_name):
                        if obj_type in assigned_types:
                            # This should not happen due to the update function
                            self.report({'ERROR'}, f"Type '{display_name}' is assigned to multiple parent collections.")
                            return
                        assigned_types[obj_type] = big_collection_name

            big_collections_created = set()
            for obj_type, objs in type_based_objects.items():
                for obj in objs:
                    if obj.name in assigned_objects:
                        continue  # Skip already assigned objects
                if obj_type in assigned_types:
                    big_collection_name = assigned_types[obj_type]
                    if big_collection_name not in big_collections_created:
                        if big_collection_name not in bpy.data.collections:
                            try:
                                big_collection = bpy.data.collections.new(big_collection_name)
                                context.scene.collection.children.link(big_collection)
                                self._steps.append(('create_collection', big_collection_name))
                                self._created_collections.add(big_collection_name)
                            except Exception as e:
                                self.report({'ERROR'}, "Failed to create collection " + f"{big_collection_name}: " + str(e))
                                return
                        else:
                            big_collection = bpy.data.collections.get(big_collection_name)
                        big_collections_created.add(big_collection_name)
                    else:
                        big_collection = bpy.data.collections.get(big_collection_name)

                    type_collection_name = collection_names[obj_type]
                    type_collection_full_name = f"{big_collection_name}_{type_collection_name}"
                    if type_collection_full_name not in bpy.data.collections:
                        try:
                            type_collection = bpy.data.collections.new(type_collection_full_name)
                            big_collection.children.link(type_collection)
                            self._steps.append(('create_collection', type_collection_full_name, big_collection_name))
                            self._created_collections.add(type_collection_full_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create collection " + f"{type_collection_full_name}: " + str(e))
                            return
                    else:
                        type_collection = bpy.data.collections.get(type_collection_full_name)

                    for obj in objs:
                        if obj.name in assigned_objects:
                            continue  # Skip already assigned objects
                        if type_collection not in obj.users_collection:
                            self._steps.append(('assign_object', type_collection_full_name, obj.name))
                            assigned_objects.add(obj.name)
                else:
                    # Assign to default type collection under scene collection
                    type_collection_name = collection_names[obj_type]
                    if type_collection_name not in bpy.data.collections:
                        try:
                            type_collection = bpy.data.collections.new(type_collection_name)
                            context.scene.collection.children.link(type_collection)
                            self._steps.append(('create_collection', type_collection_name))
                            self._created_collections.add(type_collection_name)
                        except Exception as e:
                            self.report({'ERROR'}, "Failed to create collection " + f"{type_collection_name}: " + str(e))
                            return
                    else:
                        type_collection = bpy.data.collections.get(type_collection_name)

                    for obj in objs:
                        if obj.name in assigned_objects:
                            continue  # Skip already assigned objects
                        if type_collection not in obj.users_collection:
                            self._steps.append(('assign_object', type_collection_name, obj.name))
                            assigned_objects.add(obj.name)

        else:
            # Default organization without big collections
            for obj_type, objs in type_based_objects.items():
                type_collection_name = collection_names[obj_type]
                if type_collection_name not in bpy.data.collections:
                    try:
                        type_collection = bpy.data.collections.new(type_collection_name)
                        context.scene.collection.children.link(type_collection)
                        self._steps.append(('create_collection', type_collection_name))
                        self._created_collections.add(type_collection_name)
                    except Exception as e:
                        self.report({'ERROR'}, "Failed to create collection " + f"{type_collection_name}: " + str(e))
                        return
                else:
                    type_collection = bpy.data.collections.get(type_collection_name)

                for obj in objs:
                    if obj.name in assigned_objects:
                        continue  # Skip already assigned objects
                    if type_collection not in obj.users_collection:
                        self._steps.append(('assign_object', type_collection_name, obj.name))
                        assigned_objects.add(obj.name)

        self._steps.append(('delete_empty_collections',))

    def modal(self, context, event):
        if event.type == 'ESC' and event.value == 'PRESS':
            self._cancelled = True
            self.report({'INFO'}, "Organization cancelled by user.")
            context.window_manager.progress_end()
            context.window_manager.event_timer_remove(self._timer)
            return {'CANCELLED'}

        if event.type == 'TIMER':
            try:
                elapsed = time.time() - self._start_time
                total_steps = len(self._steps)
                step_interval = self._duration / total_steps if total_steps > 0 else self._duration
                step_number = int(elapsed / step_interval)

                while step_number > self._current_step and self._current_step < total_steps:
                    step = self._steps[self._current_step]
                    if step[0] == 'create_collection':
                        if len(step) == 3:
                            collection_name, parent_name = step[1], step[2]
                            parent_collection = bpy.data.collections.get(parent_name)
                            if parent_collection and collection_name not in bpy.data.collections:
                                try:
                                    new_collection = bpy.data.collections.new(collection_name)
                                    parent_collection.children.link(new_collection)
                                except Exception as e:
                                    self.report({'ERROR'}, "Failed to create collection " + f"{collection_name}: " + str(e))
                                    return {'CANCELLED'}
                        else:
                            collection_name = step[1]
                            if collection_name not in bpy.data.collections:
                                try:
                                    new_collection = bpy.data.collections.new(collection_name)
                                    context.scene.collection.children.link(new_collection)
                                except Exception as e:
                                    self.report({'ERROR'}, "Failed to create collection " + f"{collection_name}: " + str(e))
                                    return {'CANCELLED'}
                    elif step[0] == 'assign_object':
                        collection_name, obj_name = step[1], step[2]
                        obj = bpy.data.objects.get(obj_name)
                        collection = bpy.data.collections.get(collection_name)
                        if obj and collection:
                            if collection not in obj.users_collection:
                                try:
                                    # Unlink from all other collections
                                    for col in obj.users_collection[:]:
                                        col.objects.unlink(obj)
                                    collection.objects.link(obj)
                                except Exception as e:
                                    self.report({'ERROR'}, "Failed to assign object " + f"'{obj_name}' to collection '{collection_name}': " + str(e))
                                    return {'CANCELLED'}
                    elif step[0] == 'delete_empty_collections':
                        # Collect collections to delete
                        collections_to_delete = []
                        for collection in bpy.data.collections:
                            if collection.name == context.scene.collection.name:
                                continue
                            if not collection.objects and not collection.children:
                                collections_to_delete.append(collection)

                        # Delete collected collections
                        for collection in collections_to_delete:
                            try:
                                # Unlink from all parent collections
                                for parent in bpy.data.collections:
                                    if collection.name in parent.children.keys():
                                        parent.children.unlink(collection)
                                # Remove collection
                                bpy.data.collections.remove(collection)
                            except Exception as e:
                                self.report({'ERROR'}, f"Failed to delete collection '{collection.name}': {str(e)}")

                    self._current_step += 1
                    context.window_manager.progress_update(self._current_step)

                if elapsed >= self._duration or self._current_step >= total_steps or self._cancelled:
                    context.window_manager.progress_end()
                    context.window_manager.event_timer_remove(self._timer)
                    if self._cancelled:
                        return {'CANCELLED'}
                    return {'FINISHED'}
            except Exception as e:
                self.report({'ERROR'}, "Error during organization: " + str(e))
                context.window_manager.progress_end()
                context.window_manager.event_timer_remove(self._timer)
                return {'CANCELLED'}

        return {'PASS_THROUGH'}


class ORGANIZE_OT_DeleteAllCollections(Operator):
    bl_idname = "object.delete_all_collections"
    bl_label = "Delete All Collections"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        try:
            all_objects = list(bpy.data.objects)
            for obj in all_objects:
                if context.scene.collection not in obj.users_collection:
                    context.scene.collection.objects.link(obj)
            collections_to_delete = [col for col in bpy.data.collections if col != context.scene.collection]
            for col in collections_to_delete:
                bpy.data.collections.remove(col)
            self.report({'INFO'}, "All collections deleted, objects are now in the master collection")
            return {'FINISHED'}
        except Exception as e:
            self.report({'ERROR'}, "Error deleting collections: " + str(e))
            return {'CANCELLED'}


class ORGANIZE_OT_AddParentCollection(Operator):
    bl_idname = "organize.add_parent_collection"
    bl_label = "Add Parent Collection"

    def execute(self, context):
        organize_props = context.scene.organize_properties
        new_pcoll = organize_props.parent_collections.add()
        new_pcoll.name = "Collection"
        organize_props.parent_collections_index = len(organize_props.parent_collections) - 1
        return {'FINISHED'}


class ORGANIZE_OT_RemoveParentCollection(Operator):
    bl_idname = "organize.remove_parent_collection"
    bl_label = "Remove Parent Collection"

    def execute(self, context):
        organize_props = context.scene.organize_properties
        index = organize_props.parent_collections_index
        if index < 0 or index >= len(organize_props.parent_collections):
            self.report({'ERROR'}, "Invalid parent collection index.")
            return {'CANCELLED'}
        # Reset type assignments in the collection being removed
        pcoll = organize_props.parent_collections[index]
        for obj_type, prop_name, display_name in object_types:
            setattr(pcoll, prop_name, False)
        organize_props.parent_collections.remove(index)
        if index > 0:
            organize_props.parent_collections_index -= 1
        else:
            organize_props.parent_collections_index = 0
        return {'FINISHED'}


class ORGANIZE_UL_ParentCollections(UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        pcoll = item
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(pcoll, "name", text="", emboss=False, icon='OUTLINER_COLLECTION')
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.prop(pcoll, "name", text="", emboss=False)


class ORGANIZE_OT_RefreshMaterialList(Operator):
    bl_idname = "organize.refresh_material_list"
    bl_label = "Refresh Material List"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        organize_props = context.scene.organize_properties
        # Preserve currently selected materials
        currently_selected = {mat_sel.material_name for mat_sel in organize_props.selected_materials if mat_sel.select}

        organize_props.selected_materials.clear()
        for mat in bpy.data.materials:
            mat_sel = organize_props.selected_materials.add()
            mat_sel.material_name = mat.name
            mat_sel.select = mat.name in currently_selected  # Restore selection if previously selected

        self.report({'INFO'}, "Material list refreshed.")
        return {'FINISHED'}


class ORGANIZE_UL_SelectedMaterials(UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        mat_sel = item
        if self.layout_type in {'DEFAULT', 'COMPACT'}:
            layout.prop(mat_sel, "select", text=mat_sel.material_name, toggle=True)
        elif self.layout_type in {'GRID'}:
            layout.alignment = 'CENTER'
            layout.prop(mat_sel, "select", text="", toggle=True)


class ORGANIZE_PT_Panel(Panel):
    """Creates a Panel in the 3D View Sidebar"""
    bl_label = "Auto Organize"
    bl_idname = "ORGANIZE_PT_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Auto Organize'

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        organize_props = scene.organize_properties

        # Options box
        box = layout.box()
        box.label(text="Options", icon='PREFERENCES')
        col = box.column(align=True)
        col.prop(organize_props, "enable_emission_to_light", text="Self-Emitting to Light", icon='LIGHT_DATA')
        col.prop(organize_props, "organize_selected_only", text="Selected Only", icon='RESTRICT_SELECT_OFF')

        # Big Collection toggle
        col.prop(organize_props, "big_collection_active", text="Enable Parent Collection", toggle=True, icon='OUTLINER_COLLECTION')

        # Parent-Child Organization toggle
        col.prop(organize_props, "parent_child_active", text="Enable Parent-Child Organization", toggle=True, icon='GROUP')

        # Material-Based Grouping toggle
        col.prop(organize_props, "material_based_grouping_active", text="Enable Material-Based Grouping", toggle=True, icon='MATERIAL')

        if organize_props.big_collection_active or organize_props.parent_child_active or organize_props.material_based_grouping_active:
            if organize_props.big_collection_active:
                # Parent Collections
                col.label(text="Parent Groups:", icon='OUTLINER_COLLECTION')
                row = col.row()
                row.template_list("ORGANIZE_UL_ParentCollections", "", organize_props, "parent_collections", organize_props, "parent_collections_index")

                col_operators = row.column(align=True)
                col_operators.operator("organize.add_parent_collection", icon='ADD', text="")
                col_operators.operator("organize.remove_parent_collection", icon='REMOVE', text="")

                if organize_props.parent_collections:
                    pcoll = organize_props.parent_collections[organize_props.parent_collections_index]

                    # Type assignments box
                    type_box = col.box()
                    type_box.label(text="Assign Collection", icon='OBJECT_DATA')
                    col_types = type_box.column(align=True)
                    num_columns = 2
                    rows = (len(object_types) + num_columns - 1) // num_columns
                    for row_idx in range(rows):
                        row = col_types.row(align=True)
                        for col_idx in range(num_columns):
                            idx = row_idx * num_columns + col_idx
                            if idx < len(object_types):
                                obj_type, prop_name, display_name = object_types[idx]
                                row.prop(pcoll, prop_name, text=display_name, toggle=True)

            if organize_props.material_based_grouping_active:
                # Material-Based Grouping Section
                col.label(text="Material-Based Grouping:", icon='MATERIAL')
                row = col.row(align=True)
                row.scale_x = 0.3
                row.operator("organize.refresh_material_list", icon='FILE_REFRESH', text="Refresh")
                row = col.row()
                row.template_list("ORGANIZE_UL_SelectedMaterials", "", organize_props, "selected_materials", organize_props, "material_selection_index")

        # Actions box
        action_box = layout.box()
        action_box.label(text="Actions", icon='TOOL_SETTINGS')
        action_col = action_box.column(align=True)
        action_col.operator("object.organize_collections", text="Organize Collections", icon='OUTLINER_OB_GROUP_INSTANCE')
        action_col.operator("object.delete_all_collections", text="Delete All Collections", icon='TRASH')


class ORGANIZE_OT_RegisterProperties(PropertyGroup):
    enable_emission_to_light: BoolProperty(
        name="Assign Self-Emitting Objects to Light Collection",
        description="Decide whether to put objects with self-emitting materials into the Light collection",
        default=True
    )
    organize_selected_only: BoolProperty(
        name="Organize Selected Objects Only",
        description="When enabled, only organize selected objects",
        default=False
    )
    big_collection_active: BoolProperty(
        name="Big Collection Active",
        description="Toggle to activate or deactivate big collection functionality",
        default=False
    )
    parent_child_active: BoolProperty(
        name="Parent-Child Organization Active",
        description="Toggle to activate or deactivate parent-child based organization",
        default=False
    )
    material_based_grouping_active: BoolProperty(
        name="Material-Based Grouping Active",
        description="Toggle to activate or deactivate material-based grouping of objects",
        default=False
    )
    parent_collections: CollectionProperty(type=ParentCollectionPropertyGroup)
    parent_collections_index: IntProperty()

    selected_materials: CollectionProperty(type=MaterialSelectionPropertyGroup)
    material_selection_index: IntProperty()


class ORGANIZE_AddonPreferences(AddonPreferences):
    bl_idname = __package__

    dynamic_display_time: FloatProperty(
        name="Dynamic Display Time",
        description="Time in seconds for dynamic display",
        default=3.0,
        min=0.1,
        max=10.0,
        step=1,
        precision=1
    )

    def draw(self, context):
        layout = self.layout
        layout.label(text="Auto Organize Addon Preferences")
        layout.prop(self, "dynamic_display_time", text="Dynamic Display Time (s)")


def is_single_emission_material(obj):
    if obj.type != 'MESH':
        return False
    if not obj.data:
        return False
    if not hasattr(obj.data, 'materials'):
        return False
    if not obj.data.materials:
        return False
    for mat in obj.data.materials:
        if mat is None or not mat.use_nodes:
            return False
        emission_nodes = [node for node in mat.node_tree.nodes if node.type == 'EMISSION']
        if len(emission_nodes) != 1:
            return False
        links = mat.node_tree.links
        emission_node = emission_nodes[0]
        connected = False
        for link in links:
            if hasattr(link.to_node, 'type') and link.from_node == emission_node and link.to_node.type == 'OUTPUT_MATERIAL':
                connected = True
                break
        if not connected:
            return False
    return True


def register():
    bpy.utils.register_class(ParentCollectionPropertyGroup)
    bpy.utils.register_class(MaterialSelectionPropertyGroup)
    bpy.utils.register_class(ORGANIZE_OT_OrganizeCollections)
    bpy.utils.register_class(ORGANIZE_OT_DeleteAllCollections)
    bpy.utils.register_class(ORGANIZE_OT_AddParentCollection)
    bpy.utils.register_class(ORGANIZE_OT_RemoveParentCollection)
    bpy.utils.register_class(ORGANIZE_UL_ParentCollections)
    bpy.utils.register_class(ORGANIZE_OT_RefreshMaterialList)
    bpy.utils.register_class(ORGANIZE_UL_SelectedMaterials)
    bpy.utils.register_class(ORGANIZE_PT_Panel)
    bpy.utils.register_class(ORGANIZE_OT_RegisterProperties)
    bpy.utils.register_class(ORGANIZE_AddonPreferences)
    bpy.types.Scene.organize_properties = PointerProperty(type=ORGANIZE_OT_RegisterProperties)


def unregister():
    bpy.utils.unregister_class(ParentCollectionPropertyGroup)
    bpy.utils.unregister_class(MaterialSelectionPropertyGroup)
    bpy.utils.unregister_class(ORGANIZE_OT_OrganizeCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_DeleteAllCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_AddParentCollection)
    bpy.utils.unregister_class(ORGANIZE_OT_RemoveParentCollection)
    bpy.utils.unregister_class(ORGANIZE_UL_ParentCollections)
    bpy.utils.unregister_class(ORGANIZE_OT_RefreshMaterialList)
    bpy.utils.unregister_class(ORGANIZE_UL_SelectedMaterials)
    bpy.utils.unregister_class(ORGANIZE_PT_Panel)
    bpy.utils.unregister_class(ORGANIZE_OT_RegisterProperties)
    bpy.utils.unregister_class(ORGANIZE_AddonPreferences)
    del bpy.types.Scene.organize_properties


if __name__ == "__main__":
    register()
